#include "cflironecontrol.h"

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//-Конструктор класса---------------------------------------------------------------------------------
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
CFlirOneControl::CFlirOneControl(void)
{
 ImageBuffer=new unsigned char[IMAGE_BUFFER_SIZE];
 ImageBufferSize=0;
 
 ThermalImage=new unsigned short[THERMAL_IMAGE_SIZE];
 ColorImage=new unsigned char[COLOR_IMAGE_SIZE];

 for(long n=0;n<IMAGE_WIDTH*IMAGE_HEIGHT;n++)
 {
  ThermalImage[n]=0x00;
  ColorImage[n*4+0]=128;
  ColorImage[n*4+1]=128;
  ColorImage[n*4+2]=128;
  ColorImage[n*4+3]=0xff;
 }
 
 //формируем палитру по-умолчанию
 for(long n=0;n<256;n++)
 {
  ColorMap_R[n]=(unsigned char)n;
  ColorMap_G[n]=(unsigned char)n;
  ColorMap_B[n]=(unsigned char)n;
 }
 Path=new char[255];
 strcpy(Path,""); 
 EnabledSaveColorImage=false;
 EnabledSaveThermalImage=false;
 FrameIndex=0;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//-Деструктор класса----------------------------------------------------------------------------------
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
CFlirOneControl::~CFlirOneControl()
{
 delete[](ImageBuffer);
 delete[](ThermalImage);
 delete[](ColorImage);
 delete[](Path);
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//закрытые функции класса
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//----------------------------------------------------------------------------------------------------
//вычислить crc
//----------------------------------------------------------------------------------------------------
void CFlirOneControl::CalculateCRC(unsigned short &crc,unsigned char byte)
{
 crc^=(byte<<8);
 for(unsigned char n=0;n<8;n++) 
 { 
  if (crc&0x8000) crc=(crc<<1)^0x1021;
             else crc<<=1;
 }
}

//----------------------------------------------------------------------------------------------------
//добавить строку в log-файл
//----------------------------------------------------------------------------------------------------
void CFlirOneControl::AddLog(char *string)
{
 return;  
 FILE *file=fopen("log.txt","ab");
 if (file==NULL) return;
 fprintf(file,"%s",string);
 fclose(file);
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//открытые функции класса
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//----------------------------------------------------------------------------------------------------
//задать рабочий каталог
//----------------------------------------------------------------------------------------------------
void CFlirOneControl::SetPath(char *path)
{
 delete[](Path);
 Path=new char[strlen(path)+1];
 strcpy(Path,path);
}
//----------------------------------------------------------------------------------------------------
//создать изображение
//----------------------------------------------------------------------------------------------------
bool CFlirOneControl::CreateImage(unsigned char *buffer,unsigned long size)
{
 unsigned long n;
 long x;
 long y;
 if (size+ImageBufferSize>=IMAGE_BUFFER_SIZE) ImageBufferSize=0;//начинаем сборку заново
 //копируем данные
 for(n=0;n<size;n++) ImageBuffer[n+ImageBufferSize]=buffer[n];
 ImageBufferSize+=size; 
 if (ImageBufferSize<4) return(false);

 //если в принятых данных встретились 4 специальных байта, то тоже начинаем сборку заново
 unsigned char magicbyte[4]={0xEF,0xBE,0x00,0x00};
 for(n=0;n<ImageBufferSize-4;n++)
 {
  long m;
  bool ok=true;
  for(m=0;m<4;m++)
  {
   if (ImageBuffer[n+m]==magicbyte[m]) continue;
   ok=false;
   break;
  }
  if (ok==true)//байты встретились
  {
   //переносим буфер в начало
   long size=ImageBufferSize-n-4;
   for(m=0;m<size;m++) ImageBuffer[m]=ImageBuffer[m+n+4];
   ImageBufferSize=size;
   n=0;//начинаем сканирование заново
  }
 }
 if (ImageBufferSize<24) return(false);//заголовок не собран
 unsigned long FrameSize=*((unsigned long *)(ImageBuffer+8));
 unsigned long ThermalSize=*((unsigned long *)(ImageBuffer+12));
 unsigned long JpgSize=*((unsigned long *)(ImageBuffer+16));
 unsigned long StatusSize=*((unsigned long *)(ImageBuffer+20));
 if (FrameSize+28>ImageBufferSize) return(false);//кадр ещё не собран 

 //кадр собран
 ImageBufferSize=0;
 for(n=0;n<IMAGE_WIDTH*IMAGE_HEIGHT;n++)
 {
  ThermalImage[n]=0x00;
  ColorImage[n*4+0]=128;
  ColorImage[n*4+1]=128;
  ColorImage[n*4+2]=128;
  ColorImage[n*4+3]=0xff;
 }
 //создаём тепловое изображение (164 числа в строке)
 long min=0x10000;
 long max=0x0;
 unsigned short *d_ptr=(unsigned short*)(ImageBuffer+24);
 bool enabled=true;
 for(y=0;y<ORIGINAL_IMAGE_HEIGHT;y++) 
 {  
  unsigned short crc=0;
  unsigned short id;
  unsigned short crc_package;
  //читаем заголовок
  id=*d_ptr;
  id&=0x0FFF;
  CalculateCRC(crc,(id>>8)&0xFF);
  CalculateCRC(crc,id&0xFF);
  d_ptr++;
  crc_package=*d_ptr;  
  CalculateCRC(crc,0);
  CalculateCRC(crc,0);
  d_ptr++;
  for(x=0;x<ORIGINAL_IMAGE_WIDTH/2;x++,d_ptr++)
  {   
   unsigned short value=*d_ptr;
   unsigned long offset=y+(ORIGINAL_IMAGE_WIDTH-1-x)*ORIGINAL_IMAGE_HEIGHT;   
   ThermalImage[offset]=value;
   if (value<min) min=value;
   if (value>max) max=value;
   //считаем CRC
   CalculateCRC(crc,*((unsigned char*)(d_ptr)+1));
   CalculateCRC(crc,*((unsigned char*)(d_ptr)));
  }
  if (crc!=crc_package)//CRC не совпало
  {
   enabled=false;
   break;
  }

  crc=0;
  //читаем заголовок
  id=*d_ptr;
  id&=0x0FFF;
  CalculateCRC(crc,(id>>8)&0xFF);
  CalculateCRC(crc,id&0xFF);
  d_ptr++;
  crc_package=*d_ptr;  
  CalculateCRC(crc,0);
  CalculateCRC(crc,0);
  d_ptr++;
  for(x=ORIGINAL_IMAGE_WIDTH/2;x<ORIGINAL_IMAGE_WIDTH;x++,d_ptr++)
  {   
   unsigned short value=*d_ptr;
   unsigned long offset=y+(ORIGINAL_IMAGE_WIDTH-1-x)*ORIGINAL_IMAGE_HEIGHT;   
   ThermalImage[offset]=value;
   if (value<min) min=value;
   if (value>max) max=value;
   //считаем CRC
   CalculateCRC(crc,*((unsigned char*)(d_ptr)+1));
   CalculateCRC(crc,*((unsigned char*)(d_ptr)));
  }
  if (crc!=crc_package)//CRC не совпало
  {
   enabled=false;
   break;
  }
 }
 if (enabled==true)
 {
  //создаём раскрашенное изображение
  long delta=max-min;
  if (delta==0) delta=1;
  for(x=0;x<ORIGINAL_IMAGE_WIDTH;x++)
  {
   for(y=0;y<ORIGINAL_IMAGE_HEIGHT;y++)
   {
    long offset=x+y*ORIGINAL_IMAGE_WIDTH;
    long value=ThermalImage[offset];
    value-=min;
    value=(value*255)/delta;
    offset=x+y*ORIGINAL_IMAGE_WIDTH;
    value&=0xff;
    ColorImage[offset*4+0]=ColorMap_B[value];
    ColorImage[offset*4+1]=ColorMap_G[value];
    ColorImage[offset*4+2]=ColorMap_R[value];
    ColorImage[offset*4+3]=0xff;
   }
  }
  FrameIndex++;    
  //считываем дату и время
  time_t time_main=time(NULL);
  tm *tm_main=localtime(&time_main);
  //сохраняем изображение
  char *filename=new char[strlen(Path)+1024];
  if (EnabledSaveColorImage==true)
  {
   //ищем свободный файл  	
   for(long index=1;index<1000;index++)	
   {
    sprintf(filename,"%s/image/img[%04i.%02i.%02i %02i_%02i_%02i-%03ld].tga",Path,tm_main->tm_year+1900,tm_main->tm_mon+1,tm_main->tm_mday,tm_main->tm_hour,tm_main->tm_min,tm_main->tm_sec,index);
    FILE *file=fopen(filename,"rb");
    if (file==NULL)//такого файла ещё нет
    {
     printf("Create color image %s\r\n",filename);    	
     SaveTGA(filename,IMAGE_WIDTH,IMAGE_HEIGHT,ColorImage);    	     
     break;
    }
    else fclose(file);     
   }
  }
  if (EnabledSaveThermalImage==true)
  {
   //ищем свободный файл  	
   for(long index=1;index<1000;index++)	
   {
    sprintf(filename,"%s/image/raw[%04i.%02i.%02i %02i_%02i_%02i-%03ld].raw",Path,tm_main->tm_year+1900,tm_main->tm_mon+1,tm_main->tm_mday,tm_main->tm_hour,tm_main->tm_min,tm_main->tm_sec,index);
    FILE *file=fopen(filename,"rb");
    if (file==NULL)//такого файла ещё нет
    {
     file=fopen(filename,"wb");
     if (file!=NULL)
     {
      printf("Create thermal image %s\r\n",filename);
      fwrite(ThermalImage,THERMAL_IMAGE_SIZE,sizeof(unsigned short),file);
      fclose(file);
     }
     break;
    }
    else fclose(file);     
   }
  delete[](filename);
  }
 }
 return(true);
}
//----------------------------------------------------------------------------------------------------
//загрузить карту перекодировки изображения
//----------------------------------------------------------------------------------------------------
bool CFlirOneControl::LoadColorMap(char *filename)
{ 
 FILE *file=fopen(filename,"rb");
 if (file==NULL) return(false);
 unsigned char colormap[768];
 if (fread(colormap,sizeof(unsigned char),768,file)<768)
 {
  fclose(file);
  return(false);
 }
 fclose(file);
 for(long n=0;n<256;n++)
 {
  ColorMap_R[n]=colormap[n*3+0];
  ColorMap_G[n]=colormap[n*3+1];
  ColorMap_B[n]=colormap[n*3+2];
 }
 return(true);
}
//----------------------------------------------------------------------------------------------------
//установить, сохранять ли раскрашенное изображение
//----------------------------------------------------------------------------------------------------
void CFlirOneControl::SetEnabledColorImage(bool state)
{
 EnabledSaveColorImage=state;	
}
//----------------------------------------------------------------------------------------------------
//установить, сохранять ли температурное изображение
//----------------------------------------------------------------------------------------------------
void CFlirOneControl::SetEnabledThermalImage(bool state)
{
 EnabledSaveThermalImage=state;
}

//----------------------------------------------------------------------------------------------------
//получить указатель на тепловое изображение
//----------------------------------------------------------------------------------------------------
unsigned short* CFlirOneControl::GetThermalImagePtr(unsigned long &frame_index)
{
 frame_index=FrameIndex;
 return(ThermalImage);	
}
//----------------------------------------------------------------------------------------------------
//получить указатель на раскрашенное изображение
//----------------------------------------------------------------------------------------------------
unsigned char* CFlirOneControl::GetColorImagePtr(unsigned long &frame_index)
{
 frame_index=FrameIndex;
 return(ColorImage);	
}
//-Прочее--------------------------------------------------------------------

