#ifndef CFLIRONECONTROL_H
#define CFLIRONECONTROL_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "tga.h"

#define IMAGE_BUFFER_SIZE 1048576

//исходные размеры изображения (не перевёрнутое)
#define ORIGINAL_IMAGE_WIDTH 160
#define ORIGINAL_IMAGE_HEIGHT 120

//размеры изображения после обработки и переворота в вертикальное положение
#define IMAGE_WIDTH 120
#define IMAGE_HEIGHT 160

#define THERMAL_IMAGE_SIZE (IMAGE_WIDTH*IMAGE_HEIGHT)
#define COLOR_IMAGE_SIZE (IMAGE_WIDTH*IMAGE_HEIGHT*sizeof(unsigned char)*4)

//бесконечное малое
#define EPS 0.00000001

//класс управления Flir One
class CFlirOneControl
{
 protected:
  //-Переменные класса-------------------------------------------------------  
  unsigned char *ImageBuffer;//буфер сборки пакета данных с изображением
  unsigned long ImageBufferSize;//количество данных в буфере сборки изображения
  
  unsigned short *ThermalImage;//тепловое изображение
  unsigned char *ColorImage;//раскрашенное изображение

  unsigned char ColorMap_R[256];//карта перекодировки изображения
  unsigned char ColorMap_G[256];//карта перекодировки изображения
  unsigned char ColorMap_B[256];//карта перекодировки изображения
  
  char *Path;//рабочий каталог
  
  bool EnabledSaveColorImage;//нужно ли сохранять раскрашенное изображение
  bool EnabledSaveThermalImage;//нужно ли сохранять тепловое изображение
  
  long FrameIndex;//номер текущего кадра
  //-Функции класса----------------------------------------------------------
  //-Прочее------------------------------------------------------------------
 public:
  //-Конструктор класса------------------------------------------------------
  CFlirOneControl(void);
  //-Деструктор класса-------------------------------------------------------
  ~CFlirOneControl();
  //-Переменные класса-------------------------------------------------------
  //-Функции класса----------------------------------------------------------
  void SetPath(char *path);//задать рабочий каталог
  bool CreateImage(unsigned char *buffer,unsigned long size);//создать изображение
  bool LoadColorMap(char *filename);//загрузить карту перекодировки изображения
  void SetEnabledColorImage(bool state);//установить, сохранять ли раскрашенное изображение
  void SetEnabledThermalImage(bool state);//установить, сохранять ли температурное изображение
  unsigned short* GetThermalImagePtr(unsigned long &frame_index);//получить указатель на тепловое изображение
  unsigned char* GetColorImagePtr(unsigned long &frame_index);//получить указатель на раскрашенное изображение
  //-Прочее------------------------------------------------------------------
 protected:
  void CalculateCRC(unsigned short &crc,unsigned char byte);//вычислить crc
  void AddLog(char *string);//добавить строку в log-файл
};

#endif
