#include "cmain.h"

CMain cMain;

//-Конструктор класса--------------------------------------------------------
CMain::CMain()
{
 //создаём директорию
 mkdir("./image",S_IRWXU|S_IRGRP|S_IXGRP|S_IROTH |S_IXOTH); 
 ColorImage=new unsigned char[COLOR_IMAGE_SIZE];
 ThermalImage=new unsigned short[THERMAL_IMAGE_SIZE];
 VPtr=new unsigned char[COLOR_IMAGE_SIZE*4]; 
 memset(VPtr,COLOR_IMAGE_SIZE*4,0);
 SaveAllFrame=false;
 LastSavedFrameIndex=0;
 //запускаем поток управления устройством
 cReceiveThread.StartThread(); 
}
//-Деструктор класса---------------------------------------------------------
CMain::~CMain()
{  
 if (VPtr!=NULL) delete[](VPtr);
 VPtr=NULL;	
 if (ColorImage!=NULL) delete[](ColorImage);
 ColorImage=NULL;	
 if (ThermalImage!=NULL) delete[](ThermalImage);
 ThermalImage=NULL;	
 //останавливаем поток управления устройством
 cReceiveThread.StopThread();  
}
//-Замещённые функции предка-----------------------------------------------
//-Новые функции класса------------------------------------------------------

//----------------------------------------------------------------------------------------------------
//открытие окна
//----------------------------------------------------------------------------------------------------
bool CMain::OnOpening(void)
{
 return(true);
}
//----------------------------------------------------------------------------------------------------
//закрытие окна
//----------------------------------------------------------------------------------------------------
bool CMain::OnClosing(void)
{
 return(true);	
}
//----------------------------------------------------------------------------------------------------
//перерисовать картинку
//----------------------------------------------------------------------------------------------------
void CMain::Redraw(void)
{
 //формируем изображение	
 unsigned long *s_img=reinterpret_cast<unsigned long*>(ColorImage);
 unsigned long *d_img=reinterpret_cast<unsigned long*>(VPtr);
 for(long y=0;y<IMAGE_HEIGHT*2;y++)
 { 	
  for(long x=0;x<IMAGE_WIDTH*2;x++)
  {  	
   unsigned long color=*(s_img+(x/2)+(y/2)*IMAGE_WIDTH);
   if (x==(IMAGE_WIDTH/2-5)*2 || x==(IMAGE_WIDTH/2+5)*2)
   {
    if (y>=(IMAGE_HEIGHT/2-5)*2 && y<=(IMAGE_HEIGHT/2+5)*2) color^=0x00FFFFFF;
   }   
   if (y==(IMAGE_HEIGHT/2-5)*2 || y==(IMAGE_HEIGHT/2+5)*2)
   {
    if (x>=(IMAGE_WIDTH/2-5)*2 && x<=(IMAGE_WIDTH/2+5)*2) color^=0x00FFFFFF;
   }   
   *d_img=color; 
   d_img++;
  }
 }	
 PhDim_t ImageSize;
 ImageSize.w=IMAGE_WIDTH*2;
 ImageSize.h=IMAGE_HEIGHT*2;
 int ImageBPL=ImageSize.w*4;
 PhPoint_t pos;
 pos.x=5;
 pos.y=5;
 PgDrawImage(VPtr,Pg_IMAGE_DIRECT_8888,&pos,&ImageSize,ImageBPL,0);
}
//----------------------------------------------------------------------------------------------------
//обработка таймера
//----------------------------------------------------------------------------------------------------
void CMain::OnTimer(void)
{
 unsigned long index;
 //получаем тепловое изображение
 cReceiveThread.CopyThermalImage(ThermalImage,THERMAL_IMAGE_SIZE,index);
 //получаем раскрашенное изображение
 cReceiveThread.CopyColorImage(ColorImage,COLOR_IMAGE_SIZE,index);
 //сохраняем изображение 
 if (SaveAllFrame==true)//требуется сохранить изображение
 {
  PtSetResource(ABW_PtButton_SaveAllFrame,Pt_ARG_FILL_COLOR,Pg_GREEN,0);
  if (LastSavedFrameIndex!=index)//изображение сменилось и его нужно сохранить
  {
   SaveColorImage();
   SaveThermalImage();
  }
 } 
 else PtSetResource(ABW_PtButton_SaveAllFrame,Pt_ARG_FILL_COLOR,Pg_GRAY,0);
 LastSavedFrameIndex=index;
 //вычисляем температуру
 double t=0;
 
 double PlanckR1=16528.178;
 double PlanckB=1427.5;
 double PlanckF=1.0;
 double PlanckO=-1307.0;
 double PlanckR2=0.012258549;

 double TempReflected=GetNumbericFloatValue(ABW_PtNumericFloat_TempReflected);
 double Emissivity=GetNumbericFloatValue(ABW_PtNumericFloat_Emissivity); 
 
 long point=0;
 for(long x=-5;x<5;x++)
 {
  for(long y=-5;y<5;y++)
  {
   unsigned long offset=(x+IMAGE_WIDTH/2)+(y+IMAGE_HEIGHT/2)*IMAGE_WIDTH;
   unsigned short raw14=ThermalImage[offset];
   if (raw14<=0x10000)
   {
    raw14*=4;    
    double zn=(PlanckR2*(exp(PlanckB/(TempReflected+273.15))-PlanckF));
    if (fabs(zn)<EPS) continue;
    double RAWrefl=PlanckR1/zn-PlanckO;
    double RAWobj=(raw14-(1-Emissivity)*RAWrefl)/Emissivity;
    double lgr=PlanckR1/(PlanckR2*(RAWobj+PlanckO)+PlanckF);
    if (lgr<=EPS) continue;
    double t_point=PlanckB/log(lgr)-273.15;  
    t+=t_point;	 
    point++;
   }
  }
 }
 if (point!=0)
 {
  t/=static_cast<double>(point);
  //выводим температуру
  char string[255];
  sprintf(string,"T=%.2f C",t);
  SetWidgetText(ABW_PtLabel_TValue,string); 
 }
 else SetWidgetText(ABW_PtLabel_TValue,"T=?"); 
 Redraw();	
}
//-Функции обработки сообщений класса----------------------------------------

//----------------------------------------------------------------------------------------------------
//сохранять все кадры
//----------------------------------------------------------------------------------------------------
void CMain::OnButton_SaveAllFrame(void)
{
 if (SaveAllFrame==false) SaveAllFrame=true;
                     else SaveAllFrame=false;	 
}
//----------------------------------------------------------------------------------------------------
//сохранить текущий кадр
//----------------------------------------------------------------------------------------------------
void CMain::OnButton_SaveCurrentFrame(void)
{
 SaveColorImage();	
 SaveThermalImage();
}

//-Новые функции класса------------------------------------------------------

//----------------------------------------------------------------------------------------------------
//сохранить раскрашенное изображение
//----------------------------------------------------------------------------------------------------
void CMain::SaveColorImage(void)
{
 //считываем дату и время
 time_t time_main=time(NULL);
 tm *tm_main=localtime(&time_main);
 //ищем свободный файл
 char filename[1024];
 for(long index=1;index<1000;index++)	
 {
  sprintf(filename,"./image/img[%04i.%02i.%02i %02i_%02i_%02i-%03ld].tga",tm_main->tm_year+1900,tm_main->tm_mon+1,tm_main->tm_mday,tm_main->tm_hour,tm_main->tm_min,tm_main->tm_sec,index);
  FILE *file=fopen(filename,"rb");
  if (file==NULL)//такого файла ещё нет
  {
   SaveTGA(filename,IMAGE_WIDTH,IMAGE_HEIGHT,ColorImage);    	     
   break;
  }
  else fclose(file);     
 }	
}
//----------------------------------------------------------------------------------------------------
//сохранить тепловое изображение
//----------------------------------------------------------------------------------------------------
void CMain::SaveThermalImage(void)
{
 //считываем дату и время
 time_t time_main=time(NULL);
 tm *tm_main=localtime(&time_main);
 //ищем свободный файл
 char filename[1024];
 for(long index=1;index<1000;index++)	
 {
  sprintf(filename,"./image/img[%04i.%02i.%02i %02i_%02i_%02i-%03ld].raw",tm_main->tm_year+1900,tm_main->tm_mon+1,tm_main->tm_mday,tm_main->tm_hour,tm_main->tm_min,tm_main->tm_sec,index);
  FILE *file=fopen(filename,"rb");
  if (file==NULL)//такого файла ещё нет
  {
   file=fopen(filename,"wb");
   unsigned long width=IMAGE_WIDTH;
   unsigned long height=IMAGE_HEIGHT;
   fwrite(&width,1,sizeof(unsigned long),file);
   fwrite(&height,1,sizeof(unsigned long),file);
   fwrite(ThermalImage,THERMAL_IMAGE_SIZE,sizeof(unsigned short),file);
   fclose(file);  	
   break;
  }
  else fclose(file);     
 }	
}


//-Прочее--------------------------------------------------------------------

