#ifndef CRECEIVE_THREAD
#define CRECEIVE_THREAD

//****************************************************************************************************
//Класс работы с потоком приёма данных от Flir One
//****************************************************************************************************


//****************************************************************************************************
//макроопределения
//****************************************************************************************************

#define USB_FLIR_VENDOR_ID	0x09cb
#define USB_FLIR_PRODUCT_ID	0x1996

//размер буфера URB
#define URB_BUFFER_SIZE (512<<2)

//максимальное количество точек для одного интерфейса
#define MAX_PIPE 16

//максимальное количество устройств
#define MAX_DEVICE 10

//приоритет потока приёма данных
#define RECEIVE_THREAD_PRIORITY 35

//****************************************************************************************************
//подключаемые библиотеки
//****************************************************************************************************

#include <pthread.h> //для операций с потоками и мьютексами
#include <unistd.h> //для delay
#include <unix.h> //для sigblock
#include <sys/stat.h> //для mkdir
#include <sys/usbdi.h> //для USBD

#include "cflironecontrol.h"

//****************************************************************************************************
//перечисления
//****************************************************************************************************

//****************************************************************************************************
//структуры
//****************************************************************************************************

//структура подключённых устройств
struct SDevice
{
 usbd_device *device;
 usbd_device_instance_t *instance;
 unsigned char InterfaceAddr;//адрес интерфейса
 struct SPipe
 {
  unsigned char Addr;//адрес точки
  usbd_pipe *pipe;//канал для данной точки
  usbd_urb *urb;//выделенный URB
 };
 SPipe sPipe_Array[MAX_PIPE];//список конечных точек
 long PipeAmount;//количество конечных точек
};

//структура защищённых переменных
struct SProtectedVariables
{
 pthread_mutex_t mutex_ID;//мьютекс для доступа к структуре
 SDevice sDevice_Array[MAX_DEVICE];//подключённые устройства
 long ConnectedDeviceAmount;//количество подключённых устройств
 long CurrentConnectedDeviceAmount;//количество устройств в настоящий момент времени
 CFlirOneControl cFlirOneControl;//класс управления Flir One
};

//****************************************************************************************************
//глобальные переменные
//****************************************************************************************************

//****************************************************************************************************
//прототипы функций
//****************************************************************************************************

void InsertionFlirOne(usbd_connection *connect,usbd_device_instance_t *instance);//устройство подключено
void RemovalFlirOne(usbd_connection *connect,usbd_device_instance_t *instance);//устройство отключено
void DataReceive_85(usbd_urb *urb,usbd_pipe *pipe,void *data);//принимаются данные с конечной точки 0x85
void PrintUSBDStatus(long status);//вывод сообщения об ошибке USBD

//****************************************************************************************************
//Класс работы с потоком приёма данных
//****************************************************************************************************

class CReceiveThread
{
 friend void *ReceiveThread(void *data);//поток приёма данных
 protected:
  pthread_t pthread_ID;//идентификатор потока
  pthread_mutex_t mutex_ID;//мьютекс для доступа
  bool ExitThread;//выйти из потока
  
  unsigned short *ThermoImage;//тепловое изображение
  unsigned char *ColorImage;//цветное изображение
 public:
  CReceiveThread();//конструктор
  ~CReceiveThread();//деструктор
  bool StartThread(void);//запустить поток приёма данных
  bool StopThread(void);//остановить поток приёма данных
  bool GetExitState(void);//true-требуется выход из потока
  bool CopyColorImage(unsigned char *image_ptr,unsigned long size,unsigned long &index);//скопировать раскрашенное изображение в буфер
  bool CopyThermalImage(unsigned short *image_ptr,unsigned long size,unsigned long &index);//скопировать тепловое изображение в буфер
 protected:
  void Processing(void);//основной цикл обработки (вызывается только из потока)
};

#endif

